@@ -22,6 +22,7 @@
 */
 
 #include "kwalletd.h"
+#include "kwalletd_debug.h"
 
 #include "kbetterthankdialog.h"
 #include "kwalletwizard.h"
@@ -45,19 +46,16 @@
 #include <KColorScheme>
 #include <KNotification>
 #include <KLocalizedString>
-#include <KIconLoader>
 #ifdef HAVE_GPGMEPP
 #include <gpgme++/key.h>
 #endif
 
 #include <QApplication>
-#include <QtCore/QDir>
+#include <QDir>
+#include <QIcon>
 #include <QString>
-#include <QtCore/QRegExp>
-#include <QtCore/QTimer>
-#include <QtCore/QEventLoop>
-#include <QtGui/QIcon>
-#include <QDebug>
+#include <QRegExp>
+#include <QTimer>
 
 #include <assert.h>
 
@@ -147,21 +145,21 @@ KWalletD::KWalletD()
 
     _serviceWatcher.setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
     connect(&_serviceWatcher,
-        SIGNAL(serviceOwnerChanged(QString, QString, QString)), this,
-        SLOT(slotServiceOwnerChanged(QString, QString, QString)));
+        SIGNAL(serviceOwnerChanged(QString,QString,QString)), this,
+        SLOT(slotServiceOwnerChanged(QString,QString,QString)));
 }
 
 void KWalletD::registerKWalletd4Service()
 {
     auto bus = QDBusConnection::sessionBus().interface();
-    auto reply = bus->registerService(QLatin1String("org.kde.kwalletd"), QDBusConnectionInterface::QueueService);
+    auto reply = bus->registerService(QStringLiteral("org.kde.kwalletd"), QDBusConnectionInterface::QueueService);
     if (reply.isValid() && (reply.value() == QDBusConnectionInterface::ServiceQueued)) {
-      QDBusInterface _kde_kwalletd4("org.kde.kwalletd", "/MainApplication", "org.kde.KApplication");
+      QDBusInterface _kde_kwalletd4(QStringLiteral("org.kde.kwalletd"), QStringLiteral("/MainApplication"), QStringLiteral("org.kde.KApplication"));
       if (_kde_kwalletd4.isValid()) {
-	auto qreply = _kde_kwalletd4.call("quit");
+    auto qreply = _kde_kwalletd4.call(QStringLiteral("quit"));
       }
     }
-    QDBusConnection::sessionBus().registerObject(QLatin1String("/modules/kwalletd"), this);
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/modules/kwalletd"), this);
 }
 
 KWalletD::~KWalletD()
@@ -180,14 +178,14 @@ void KWalletD::connectToScreenSaver()
     screensaver = new QDBusInterface("org.freedesktop.ScreenSaver",
         "/ScreenSaver", "org.freedesktop.ScreenSaver");
     if (!screensaver->isValid()) {
-        qDebug() << "Service org.freedesktop.ScreenSaver not found. Retrying in 10 seconds...";
+        qCDebug(KWALLETD_LOG) << "Service org.freedesktop.ScreenSaver not found. Retrying in 10 seconds...";
         // keep attempting every 10 seconds
         QTimer::singleShot(10000, this, SLOT(connectToScreenSaver()));
     }
     else {
         connect(screensaver, SIGNAL(ActiveChanged(bool)),
             SLOT(screenSaverChanged(bool)));
-        qDebug() << "connected to screen saver service.";
+        qCDebug(KWALLETD_LOG) << "connected to screen saver service.";
     }
 }
 #endif
@@ -217,6 +215,8 @@ QPair<int, KWallet::Backend*> KWalletD::findWallet(
     return qMakePair(-1, static_cast<KWallet::Backend*>(nullptr));
 }
 
+const QRegularExpression walletRegex(
+            QStringLiteral("^[\\w\\^\\&\\'\\@\\{\\}\\[\\]\\,\\$\\=\\!\\-\\#\\(\\)\\%\\.\\+\\_\\s]+$"));
 bool KWalletD::_processing = false;
 
 void KWalletD::processTransactions()
@@ -333,8 +333,7 @@ int KWalletD::open(const QString& wallet, qlonglong wId, const QString& appid)
         return -1;
     }
 
-    if (!QRegExp(QString::fromLatin1("^[\\w\\^\\&\\'\\@\\{\\}\\[\\]\\,\\$\\=\\!\\-\\#\\(\\)\\%\\."
-                 "\\+\\_\\s]+$")).exactMatch(wallet)) {
+    if (!walletRegex.match(wallet).hasMatch()) {
         return -1;
     }
 
@@ -365,8 +364,7 @@ int KWalletD::openAsync(const QString& wallet, qlonglong wId,
         return -1;
     }
 
-    if (!QRegExp(QString::fromLatin1("^[\\w\\^\\&\\'\\@\\{\\}\\[\\]\\,\\$\\=\\!\\-\\#\\(\\)\\%\\."
-                 "\\+\\_\\s]+$")).exactMatch(wallet)) {
+    if (!walletRegex.match(wallet).hasMatch()) {
         return -1;
     }
 
@@ -380,7 +378,7 @@ int KWalletD::openAsync(const QString& wallet, qlonglong wId,
     xact->tType = KWalletTransaction::Open;
     xact->isPath = false;
     if (handleSession) {
-        qDebug() << "openAsync for " << message().service();
+        qCDebug(KWALLETD_LOG) << "openAsync for " << message().service();
         _serviceWatcher.setConnection(connection());
         _serviceWatcher.addWatchedService(message().service());
         xact->service = message().service();
@@ -408,7 +406,7 @@ int KWalletD::openPathAsync(const QString& path, qlonglong wId,
     xact->tType = KWalletTransaction::Open;
     xact->isPath = true;
     if (handleSession) {
-        qDebug() << "openPathAsync " << message().service();
+        qCDebug(KWALLETD_LOG) << "openPathAsync " << message().service();
         _serviceWatcher.setConnection(connection());
         _serviceWatcher.addWatchedService(message().service());
         xact->service = message().service();
@@ -424,8 +422,9 @@ void KWalletD::setupDialog(
     QWidget* dialog, WId wId, const QString& appid, bool modal)
 {
     if (wId != 0) {
-        KWindowSystem::setMainWindow(
-            dialog, wId); // correct, set dialog parent
+        // correct, set dialog parent
+        dialog->setAttribute(Qt::WA_NativeWindow, true);
+        KWindowSystem::setMainWindow(dialog->windowHandle(), wId);
     }
     else {
         if (appid.isEmpty()) {
@@ -579,7 +578,7 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet,
     int rc = walletInfo.first;
     if (rc == -1) {
         if (_wallets.count() > 20) {
-            qDebug() << "Too many wallets open.";
+            qCDebug(KWALLETD_LOG) << "Too many wallets open.";
             return -1;
         }
 
@@ -634,9 +633,7 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet,
                     //              kpd->setButtonGuiItem(KDialog::Ok,KGuiItem(
                     //              i18n( "&Open" ), "wallet-open"));
                     kpd->setWindowTitle(i18n("KDE Wallet Service"));
-                    kpd->setPixmap(
-                        KIconLoader::global()->loadIcon(QStringLiteral("kwalletmanager"),
-                            KIconLoader::Desktop, KIconLoader::SizeHuge));
+                    kpd->setIcon(QIcon::fromTheme(QStringLiteral("kwalletmanager")));
                     if (w != KWindowSystem::activeWindow() && w != 0L) {
                         // If the dialog is modal to a minimized window it
                         // might not be visible
@@ -681,12 +678,13 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet,
                             password = kpd->password();
                             int rc = b->open(password.toUtf8());
                             if (!b->isOpen()) {
+                                const auto errorStr = KWallet::Backend::openRCToString(rc);
+                                qCWarning(KWALLETD_LOG) << "Failed to open wallet" << wallet << errorStr;
                                 kpd->setPrompt(i18n(
                                     "<qt>Error opening the wallet "
                                     "'<b>%1</b>'. Please try again.<br "
                                     "/>(Error code %2: %3)</qt>",
-                                    wallet.toHtmlEscaped(), rc,
-                                    KWallet::Backend::openRCToString(rc)));
+                                    wallet.toHtmlEscaped(), rc, errorStr));
                                 kpd->setPassword(QLatin1String(""));
                             }
                         }
@@ -780,9 +778,7 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet,
                 kpd->setWindowTitle(i18n("KDE Wallet Service"));
                 // KF5 FIXME what should we use now instead of this:
                 //              kpd->setButtonGuiItem(KDialog::Ok,KGuiItem(i18n("C&reate"),"document-new"));
-                kpd->setPixmap(
-                    KIconLoader::global()->loadIcon(QStringLiteral("kwalletmanager"),
-                        KIconLoader::Desktop, KIconLoader::SizeHuge));
+                kpd->setIcon(QIcon::fromTheme(QStringLiteral("kwalletmanager")));
                 while (!b->isOpen()) {
                     setupDialog(kpd, w, appid, modal);
                     if (kpd->exec() == QDialog::Accepted) {
@@ -837,7 +833,7 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet,
         emit walletOpened(wallet);
         if (_wallets.count() == 1 && _launchManager) {
             KToolInvocation::startServiceByDesktopName(
-                QStringLiteral("kwalletmanager-kwalletd"));
+                QStringLiteral("kwalletmanager5-kwalletd"));
         }
     }
     else {
@@ -1177,7 +1173,8 @@ QStringList KWalletD::wallets() const
 
     dir.setFilter(QDir::Files | QDir::Hidden);
 
-    foreach (const QFileInfo& fi, dir.entryInfoList()) {
+    const auto list = dir.entryInfoList();
+    for (const QFileInfo& fi : list) {
         QString fn = fi.fileName();
         if (fn.endsWith(QLatin1String(".kwl"))) {
             fn.truncate(fn.length() - 4);
@@ -1300,6 +1297,7 @@ QByteArray KWalletD::readMap(int handle, const QString& folder,
     return QByteArray();
 }
 
+#if KWALLET_BUILD_DEPRECATED_SINCE(5, 72)
 QVariantMap KWalletD::readMapList(int handle, const QString& folder,
     const QString& key, const QString& appid)
 {
@@ -1308,7 +1306,8 @@ QVariantMap KWalletD::readMapList(int handle, const QString& folder,
     if ((b = getWallet(appid, handle))) {
         b->setFolder(folder);
         QVariantMap rc;
-        foreach (KWallet::Entry* entry, b->readEntryList(key)) {
+        const auto lst = b->readEntryList(key);
+        for (KWallet::Entry* entry : lst) {
             if (entry->type() == KWallet::Wallet::Map) {
                 rc.insert(entry->key(), entry->map());
             }
@@ -1318,6 +1317,25 @@ QVariantMap KWalletD::readMapList(int handle, const QString& folder,
 
     return QVariantMap();
 }
+#endif
+
+QVariantMap KWalletD::mapList(int handle, const QString &folder, const QString &appid)
+{
+    QVariantMap rc;
+
+    KWallet::Backend *backend = getWallet(appid, handle);
+    if (backend) {
+        backend->setFolder(folder);
+        const QList<KWallet::Entry *> lst = backend->entriesList();
+        for (KWallet::Entry *entry : lst) {
+            if (entry->type() == KWallet::Wallet::Map) {
+                rc.insert(entry->key(), entry->map());
+            }
+        }
+    }
+
+    return rc;
+}
 
 QByteArray KWalletD::readEntry(int handle, const QString& folder,
     const QString& key, const QString& appid)
@@ -1335,6 +1353,7 @@ QByteArray KWalletD::readEntry(int handle, const QString& folder,
     return QByteArray();
 }
 
+#if KWALLET_BUILD_DEPRECATED_SINCE(5, 72)
 QVariantMap KWalletD::readEntryList(int handle, const QString& folder,
     const QString& key, const QString& appid)
 {
@@ -1343,7 +1362,8 @@ QVariantMap KWalletD::readEntryList(int handle, const QString& folder,
     if ((b = getWallet(appid, handle))) {
         b->setFolder(folder);
         QVariantMap rc;
-        foreach (KWallet::Entry* entry, b->readEntryList(key)) {
+        const auto lst = b->readEntryList(key);
+        for (KWallet::Entry* entry : lst) {
             rc.insert(entry->key(), entry->value());
         }
         return rc;
@@ -1351,6 +1371,24 @@ QVariantMap KWalletD::readEntryList(int handle, const QString& folder,
 
     return QVariantMap();
 }
+#endif
+
+QVariantMap KWalletD::entriesList(int handle, const QString &folder, const QString &appid)
+{
+
+    QVariantMap rc;
+
+    KWallet::Backend *backend = getWallet(appid, handle);
+    if (backend) {
+        backend->setFolder(folder);
+        const QList<KWallet::Entry *> lst = backend->entriesList();
+        for (KWallet::Entry *entry : lst) {
+            rc.insert(entry->key(), entry->value());
+        }
+    }
+
+    return rc;
+}
 
 QStringList KWalletD::entryList(
     int handle, const QString& folder, const QString& appid)
@@ -1381,6 +1419,7 @@ QString KWalletD::readPassword(int handle, const QString& folder,
     return QString();
 }
 
+#if KWALLET_BUILD_DEPRECATED_SINCE(5, 72)
 QVariantMap KWalletD::readPasswordList(int handle, const QString& folder,
     const QString& key, const QString& appid)
 {
@@ -1389,7 +1428,8 @@ QVariantMap KWalletD::readPasswordList(int handle, const QString& folder,
     if ((b = getWallet(appid, handle))) {
         b->setFolder(folder);
         QVariantMap rc;
-        foreach (KWallet::Entry* entry, b->readEntryList(key)) {
+        const auto lst = b->readEntryList(key);
+        for (KWallet::Entry* entry : lst) {
             if (entry->type() == KWallet::Wallet::Password) {
                 rc.insert(entry->key(), entry->password());
             }
@@ -1399,6 +1439,25 @@ QVariantMap KWalletD::readPasswordList(int handle, const QString& folder,
 
     return QVariantMap();
 }
+#endif
+
+QVariantMap KWalletD::passwordList(int handle, const QString &folder, const QString &appid)
+{
+    QVariantMap rc;
+
+    KWallet::Backend *backend = getWallet(appid, handle);
+    if (backend) {
+        backend->setFolder(folder);
+        const QList<KWallet::Entry *> lst = backend->entriesList();
+        for (KWallet::Entry *entry : lst) {
+            if (entry->type() == KWallet::Wallet::Password) {
+                rc.insert(entry->key(), entry->password());
+            }
+        }
+    }
+
+    return rc;
+}
 
 int KWalletD::writeMap(int handle, const QString& folder, const QString& key,
     const QByteArray& value, const QString& appid)
@@ -1538,7 +1597,7 @@ void KWalletD::slotServiceOwnerChanged(
     const QString& name, const QString& oldOwner, const QString& newOwner)
 {
     Q_UNUSED(name);
-    qDebug() << "slotServiceOwnerChanged " << name << ", " << oldOwner << ", "
+    qCDebug(KWALLETD_LOG) << "slotServiceOwnerChanged " << name << ", " << oldOwner << ", "
              << newOwner;
 
     if (!newOwner.isEmpty()) {
@@ -1549,11 +1608,11 @@ void KWalletD::slotServiceOwnerChanged(
     // all sessions. As an application can basically open wallets
     // with several appids, we can't stop if we found one.
     QString service(oldOwner);
-    QList<KWalletAppHandlePair> sessremove(_sessions.findSessions(service));
+    const QList<KWalletAppHandlePair> sessremove(_sessions.findSessions(service));
     KWallet::Backend* b = nullptr;
 
     // check all sessions for wallets to close
-    Q_FOREACH (const KWalletAppHandlePair& s, sessremove) {
+    for (const KWalletAppHandlePair& s : sessremove) {
         b = getWallet(s.first, s.second);
         if (b) {
             b->deref();
@@ -1562,7 +1621,7 @@ void KWalletD::slotServiceOwnerChanged(
     }
 
     // remove all the sessions in case they aren't gone yet
-    Q_FOREACH (const KWalletAppHandlePair& s, sessremove) {
+    for (const KWalletAppHandlePair& s : sessremove) {
         _sessions.removeSession(s.first, service, s.second);
     }
 
@@ -1581,7 +1640,7 @@ void KWalletD::slotServiceOwnerChanged(
     // mark it as cancelled.
     if (_curtrans && _curtrans->tType == KWalletTransaction::Open
         && _curtrans->service == oldOwner) {
-        qDebug() << "Cancelling current transaction!";
+        qCDebug(KWALLETD_LOG) << "Cancelling current transaction!";
         _curtrans->cancelled = true;
     }
     _serviceWatcher.removeWatchedService(oldOwner);
@@ -1687,7 +1746,8 @@ void KWalletD::emitFolderUpdated(const QString& wallet, const QString& folder)
 void KWalletD::emitWalletListDirty()
 {
     const QStringList walletsInDisk = wallets();
-    foreach (auto i, _wallets.values()) {
+    const auto lst = _wallets.values();
+    for (auto i : lst) {
         if (!walletsInDisk.contains(i->walletName())) {
             internalClose(i, _wallets.key(i), true, false);
         }
@@ -1870,8 +1930,7 @@ int KWalletD::pamOpen(
         return -1;
     }
 
-    if (!QRegExp(QString::fromLatin1("^[\\w\\^\\&\\'\\@\\{\\}\\[\\]\\,\\$\\=\\!\\-\\#\\(\\)\\%\\."
-                 "\\+\\_\\s]+$")).exactMatch(wallet)) {
+    if (!walletRegex.match(wallet).hasMatch()) {
         return -1;
     }
 
@@ -1919,7 +1978,7 @@ int KWalletD::pamOpen(
     emit walletOpened(wallet);
 
     if (_wallets.count() == 1 && _launchManager) {
-        KToolInvocation::startServiceByDesktopName(QStringLiteral("kwalletmanager-kwalletd"));
+        KToolInvocation::startServiceByDesktopName(QStringLiteral("kwalletmanager5-kwalletd"));
     }
 
     return handle;
